/*!
 * @eslint-sets/eslint-config-basic v4.0.0-beta.3
 * Eslint config basic sets
 * (c) 2021-2023 saqqdy<https://github.com/saqqdy> 
 * Released under the MIT License.
 */
'use strict';

const config = {
  root: true,
  env: {
    node: true,
    browser: true,
    commonjs: true,
    es6: true
  },
  parserOptions: {
    ecmaVersion: "latest",
    sourceType: "module",
    ecmaFeatures: {
      experimentalObjectRestSpread: true,
      experimentalDecorators: true,
      jsx: true
    }
  },
  ignorePatterns: [
    "*.min.*",
    "dist",
    "LICENSE*",
    "output",
    "coverage",
    "public",
    "temp",
    "packages-lock.json",
    "pnpm-lock.yaml",
    "yarn.lock",
    "__snapshots__",
    "!.github",
    "!.vitepress",
    "!.vuepress"
  ],
  plugins: ["html", "unicorn", "prettier"],
  extends: [
    "./standard",
    "plugin:import/recommended",
    "plugin:eslint-comments/recommended",
    "plugin:jsonc/recommended-with-jsonc",
    "plugin:yml/standard",
    "plugin:markdown/recommended",
    "prettier"
  ],
  settings: {
    "import/resolver": {
      node: { extensions: [".js", ".mjs", ".cjs"] }
    }
  },
  rules: {
    "no-console": ["warn", { allow: ["warn", "error", "info"] }],
    "no-debugger": 1,
    semi: [2, "never"],
    quotes: ["error", "single"],
    "comma-dangle": "off",
    camelcase: ["error", { properties: "never" }],
    "no-var": "error",
    "no-empty": ["error", { allowEmptyCatch: true }],
    "no-void": "error",
    "one-var": [
      "warn",
      {
        var: "always",
        let: "always",
        const: "never"
      }
    ],
    "no-throw-literal": "off",
    "no-new-wrappers": 2,
    "no-useless-escape": "off",
    "no-redeclare": 2,
    "no-tabs": "off",
    indent: "off",
    "no-mixed-spaces-and-tabs": ["error", "smart-tabs"],
    "space-before-function-paren": [0, "always"],
    "object-shorthand": 2,
    "no-unused-vars": [
      2,
      {
        args: "none",
        caughtErrors: "none",
        ignoreRestSiblings: true,
        vars: "all",
        argsIgnorePattern: "^h$",
        varsIgnorePattern: "^h$"
      }
    ],
    "no-dupe-keys": 2,
    "no-func-assign": 2,
    "valid-typeof": 2,
    "no-shadow": "off",
    "no-prototype-builtins": "off",
    "no-undef": 2,
    "no-irregular-whitespace": 1,
    "array-callback-return": "error",
    "block-scoped-var": "error",
    "consistent-return": "off",
    complexity: ["off", 11],
    eqeqeq: ["error", "smart"],
    "no-alert": "warn",
    "no-case-declarations": "error",
    "no-multi-spaces": "error",
    "no-multi-str": "error",
    "no-with": "error",
    "vars-on-top": "error",
    "require-await": "off",
    "no-return-assign": "off",
    "operator-linebreak": "off",
    "no-use-before-define": ["error", { functions: false, classes: false, variables: true }],
    "no-template-curly-in-string": "off",
    "no-extra-semi": "off",
    "no-labels": 0,
    "eslint-comments/disable-enable-pair": "off",
    "import/order": "error",
    "import/first": "off",
    "import/no-duplicates": "error",
    "import/no-mutable-exports": "error",
    "import/no-unresolved": "off",
    "import/no-absolute-path": "off",
    "import/no-named-as-default": "off",
    "import/no-named-as-default-member": "off",
    "n/no-callback-literal": "off",
    "prettier/prettier": "error",
    "sort-imports": [
      "error",
      {
        ignoreCase: false,
        ignoreDeclarationSort: true,
        ignoreMemberSort: false,
        memberSyntaxSortOrder: ["none", "all", "multiple", "single"],
        allowSeparatedGroups: false
      }
    ],
    "yml/quotes": ["error", { prefer: "single", avoidEscape: false }],
    "yml/no-empty-document": "off",
    "unicorn/error-message": "error",
    "unicorn/escape-case": "error",
    "unicorn/no-array-instanceof": "error",
    "unicorn/no-new-buffer": "error",
    "unicorn/no-unsafe-regex": "off",
    "unicorn/number-literal-case": "error",
    "unicorn/prefer-exponentiation-operator": "error",
    "unicorn/prefer-includes": "error",
    "unicorn/prefer-starts-ends-with": "error",
    "unicorn/prefer-text-content": "error",
    "unicorn/prefer-type-error": "error",
    "unicorn/throw-new-error": "error"
  },
  globals: {
    document: "readonly",
    navigator: "readonly",
    window: "readonly"
  },
  overrides: [
    {
      files: ["*.json", "*.json5"],
      parser: "jsonc-eslint-parser",
      rules: {
        quotes: ["error", "double"],
        "quote-props": ["error", "always"],
        "comma-dangle": ["error", "never"]
      }
    },
    {
      files: ["*.yaml", "*.yml"],
      parser: "yaml-eslint-parser",
      rules: {
        "spaced-comment": "off"
      }
    },
    {
      files: ["package.json"],
      parser: "jsonc-eslint-parser",
      rules: {
        "jsonc/sort-keys": [
          "error",
          {
            pathPattern: "^$",
            order: [
              "name",
              "description",
              "type",
              "version",
              "private",
              "packageManager",
              "bin",
              "main",
              "module",
              "unpkg",
              "jsdelivr",
              "types",
              "exports",
              "files",
              "scripts",
              "dependencies",
              "devDependencies",
              "peerDependencies",
              "peerDependenciesMeta",
              "optionalDependencies",
              "sideEffects",
              "keywords",
              "license",
              "author",
              "repository",
              "funding",
              "husky",
              "lint-staged",
              "eslintConfig"
            ]
          },
          {
            pathPattern: "^(?:dev|peer|optional|bundled)?[Dd]ependencies$",
            order: { type: "asc" }
          }
        ]
      }
    },
    {
      files: ["*.d.ts"],
      rules: {
        "import/no-duplicates": "off"
      }
    },
    {
      files: ["*.js"],
      rules: {}
    },
    {
      files: ["scripts/**/*.*", "cli.*"],
      rules: {
        "no-console": "off"
      }
    },
    {
      files: ["*.test.ts", "*.test.js", "*.spec.ts", "*.spec.js"],
      rules: {
        "no-unused-expressions": "off"
      }
    },
    {
      files: ["**/*.md/*.*"],
      rules: {
        "import/no-unresolved": "off",
        "no-alert": "off",
        "no-console": "off",
        "no-restricted-imports": "off",
        "no-undef": "off",
        "no-unused-expressions": "off",
        "no-unused-vars": "off"
      }
    }
  ]
};

module.exports = config;
